
From: Manuel Estrada Sainz <ranty@debian.org>

In its current form request_firmware sleeps for too long on the system's
common workqueue, and using a private workqueue as previously proposed is
not optimal.  This patch creates one kernel_thread for each
request_firmware_async() invocation which dies once the job is done.



 drivers/base/firmware_class.c |   21 ++++++++++++++++-----
 1 files changed, 16 insertions(+), 5 deletions(-)

diff -puN drivers/base/firmware_class.c~firmware-kernel_thread-on-demand drivers/base/firmware_class.c
--- 25/drivers/base/firmware_class.c~firmware-kernel_thread-on-demand	2003-10-21 11:27:27.000000000 -0700
+++ 25-akpm/drivers/base/firmware_class.c	2003-10-21 11:27:27.000000000 -0700
@@ -415,18 +415,22 @@ struct firmware_work {
 	void (*cont)(const struct firmware *fw, void *context);
 };
 
-static void
+static int
 request_firmware_work_func(void *arg)
 {
 	struct firmware_work *fw_work = arg;
 	const struct firmware *fw;
-	if (!arg)
-		return;
+	if (!arg) {
+		WARN_ON(1);
+		return 0;
+	}
+	daemonize("firmware/%s", fw_work->name);
 	request_firmware(&fw, fw_work->name, fw_work->device);
 	fw_work->cont(fw, fw_work->context);
 	release_firmware(fw);
 	module_put(fw_work->module);
 	kfree(fw_work);
+	return 0;
 }
 
 /**
@@ -451,6 +455,8 @@ request_firmware_nowait(
 {
 	struct firmware_work *fw_work = kmalloc(sizeof (struct firmware_work),
 						GFP_ATOMIC);
+	int ret;
+
 	if (!fw_work)
 		return -ENOMEM;
 	if (!try_module_get(module)) {
@@ -465,9 +471,14 @@ request_firmware_nowait(
 		.context = context,
 		.cont = cont,
 	};
-	INIT_WORK(&fw_work->work, request_firmware_work_func, fw_work);
 
-	schedule_work(&fw_work->work);
+	ret = kernel_thread(request_firmware_work_func, fw_work,
+			    CLONE_FS | CLONE_FILES);
+
+	if (ret < 0) {
+		fw_work->cont(NULL, fw_work->context);
+		return ret;
+	}
 	return 0;
 }
 

_
